<?php

/*
 *     doughnutwedding.com
 *     Copyright (C) 2017  http://doughnutwedding.com eric@doughnutwedding.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\DoughnutWeddingBundle\Controller;

use Pagerfanta\Adapter\ArrayAdapter;
use Pagerfanta\Pagerfanta;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\{
  Method,
  Route
};
use Sikofitt\DoughnutWeddingBundle\Entity\{
  Image,
  ImageComment,
  User
};
use Sikofitt\DoughnutWeddingBundle\Form\GalleryUploadType;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\{
  JsonResponse,
  RedirectResponse,
  Request,
  Response
};
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Security\{
  Core\Authentication\Token\TokenInterface,
  Csrf\CsrfToken
};

/**
 * Class ImageController.
 *
 * @Route("/gallery")
 */
class GalleryController extends Controller
{
    /**
     * @Route("/")
     * @Method({"GET"})
     */
    public function indexAction(Request $request): Response
    {
        $images = $this
          ->get('doctrine.orm.entity_manager')
          ->getRepository('SikofittDoughnutWeddingBundle:Image')
          ->findAllSortedByUpdatedAtDesc();

        $chunks = array_chunk($images, 12);
        $adapter = new ArrayAdapter($images);
        $pager = new Pagerfanta($adapter);
        $pager->setMaxPerPage(12);

        return $this->render('SikofittDoughnutWeddingBundle:Image:index.html.twig', [
          'images' => $chunks[0] ?? $images,
          'pager' => $pager,
        ]);
    }

    /**
     * @Route("/page/{number}", defaults={"number":1}, name="gallery_page_view", requirements={"number":"\d+"})
     *
     * @param \Symfony\Component\HttpFoundation\Request $request
     * @param int                                       $number
     *
     * @return Response
     */
    public function pageAction(Request $request, int $number)
    {
        $images = $this
        ->get('doctrine.orm.entity_manager')
        ->getRepository('SikofittDoughnutWeddingBundle:Image')
        ->findAllSortedByUpdatedAtDesc();

        $adapter = new ArrayAdapter($images);
        $pager = new Pagerfanta($adapter);
        $chunks = array_chunk($images, 12);
        $pager->setMaxPerPage(12);
        $pager->setCurrentPage($number);

        return $this->render('SikofittDoughnutWeddingBundle:Image:index.html.twig', [
          'images' => $chunks[$number - 1] ?? $images,
          'pager' => $pager,
        ]);
    }

    /**
     * @Route("/upload")
     */
    public function uploadAction(Request $request, Response $response = null)
    {
        if (false === $this->isGranted('IS_AUTHENTICATED_FULLY')) {
            $this->addFlash('error', 'You must be logged in to upload photos.');
        }

        $image = new Image();
        $image->setUser($request->getSession()->get('user'));
        $form = $this->get('form.factory')->create(GalleryUploadType::class, $image);
        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            /** @var \Sikofitt\DoughnutWeddingBundle\Entity\User $user */
            $user = $this->getUser();
            $image->setUser($user);

            $em = $this->get('doctrine.orm.entity_manager');
            $em->persist($image);
            $em->flush();

            $baseName = $image
              ->getImageFile()
              ->getBasename(
                sprintf('.%s', $image->getImageFile()->getExtension())
              );

            return new RedirectResponse(
              $this
                ->get('router')
                ->generate('view_gallery_image_by_hash', [
                  'imageHash' => $baseName,
                ])
            );
        }

        return $this->render('SikofittDoughnutWeddingBundle:Image:upload.html.twig', [
            'form' => $form->createView(),
        ]);
    }

    /**
     * @Route("/ajax")
     * @Method({"POST"})
     */
    public function ajaxCommentAction(Request $request): JsonResponse
    {
        if (false === $request->getSession()->has('_security_secured_area')) {
            return new JsonResponse([
              'status' => 403,
              'message' => 'You must be logged in to post a comment',
            ], 403);
        }

        /** @var \Symfony\Component\Security\Core\Authentication\Token\TokenInterface $data */
        $data = unserialize(
          $request->getSession()->get('_security_secured_area'), [
            TokenInterface::class,
          ]);

        $tokenManager = $this->get('security.csrf.token_manager');
        $fileName = $request->request->get('file');
        $token = $request->request->get('_token');
        $csrfToken = new CsrfToken($fileName, $token);

        if (false === $tokenManager->isTokenValid($csrfToken)) {
            return new JsonResponse(['status' => Response::HTTP_EXPECTATION_FAILED, 'message' => 'Invalid Token'], Response::HTTP_EXPECTATION_FAILED);
        }

        if (false === $data->getUser() instanceof User) {
            return new JsonResponse([
            'status' => 403,
            'message' => 'You must be logged in to post a comment',
          ], 403);
        }

        $em = $this->get('doctrine.orm.entity_manager');
        $user = $em
          ->getRepository('SikofittDoughnutWeddingBundle:User')
          ->findOneBy(['id' => $data->getUser()->getId()]);

        $comment = $request->request->get('comment');
        $fileId = $request->request->get('id');

        $image = $em
          ->getRepository('SikofittDoughnutWeddingBundle:Image')
          ->findOneBy(['id' => $fileId]);

        $newComment = new ImageComment();
        $newComment->setUser($user)
          ->setComment($comment)
            ->setImage($image)
            ->setIsChild(false)
            ->setCreated(new \DateTime('now'))
            ->setChildComments(null);

        $em->persist($newComment);
        $em->flush();

        $commentHtml = $this->renderView('gallery_comment.html.twig', ['comment' => $newComment]);

        return new JsonResponse([
          'id' => $user->getId(),
          'name' => $user->getName(),
          'date_short' => date('m/d/y').' at '.date('g:ia'),
          'date_long' => date('l F jS, Y').' at '.date('g:ia'),
          'status' => Response::HTTP_OK,
          'html' => $commentHtml,
        ]);
    }

    /**
     * @Route("/{imageHash}", name="view_gallery_image_by_hash", requirements={"imageHash":"\w{32,32}"})
     * @Method({"GET","HEAD"})
     *
     * @param \Symfony\Component\HttpFoundation\Request $request
     * @param string                                    $imageHash
     *
     * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
     *
     * @return Response
     */
    public function viewImageAction(Request $request, string $imageHash): ?Response
    {
        $image = $this
          ->get('doctrine.orm.entity_manager')
          ->createQueryBuilder()
          ->select('i')
          ->from(Image::class, 'i')
          ->where('i.imageName LIKE :imageHash')
          ->setParameter('imageHash', $imageHash.'%')
          ->getQuery()
          ->getOneOrNullResult();

        if (null !== $image) {
            return $this
              ->render('@SikofittDoughnutWedding/Image/view_image.html.twig', [
                'image' => $image,
              ]);
        }

        throw new NotFoundHttpException();
    }

    /**
     * @Route("/tag/{tag}/{number}", name="gallery_tag_action", defaults={"number":1}, requirements={"number":"\d+"})
     *
     * @param \Symfony\Component\HttpFoundation\Request $request
     * @param string                                    $tag
     * @param int                                       $number
     *
     * @return Response
     */
    public function tagAction(Request $request, string $tag, int $number): Response
    {
        $images = $this
          ->get('doctrine.orm.entity_manager')
          ->getRepository('SikofittDoughnutWeddingBundle:Image')
          ->findImageByTag($tag);
        $adapter = new ArrayAdapter($images);
        $pager = new Pagerfanta($adapter);
        $pager->setMaxPerPage(12);
        $chunks = array_chunk($images, 12);

        return $this
          ->render('SikofittDoughnutWeddingBundle:Image:index.html.twig', [
            'pager' => $pager,
            'tag' => $tag,
            'images' => $chunks[$number - 1] ?? $images,
          ]);
    }

    /**
     * @Route("/category/{category}/{number}", name="gallery_category_action", defaults={"number":1}, requirements={"number":"\d+"})
     *
     * @param \Symfony\Component\HttpFoundation\Request $request
     * @param string                                    $category
     * @param int                                       $number
     *                                                            The current page
     *
     * @return Response
     */
    public function categoryAction(Request $request, string $category, int $number): Response
    {
        $images = $this
          ->get('doctrine.orm.entity_manager')
          ->getRepository('SikofittDoughnutWeddingBundle:Image')
          ->findImageByCategory($category);

        $adapter = new ArrayAdapter($images);
        $pager = new Pagerfanta($adapter);
        $pager->setMaxPerPage(12);
        $chunks = array_chunk($images, 12);

        return $this
          ->render('SikofittDoughnutWeddingBundle:Image:index.html.twig', [
            'pager' => $pager,
            'category' => $category,
            'images' => $chunks[$number - 1] ?? $images,
          ]);
    }
}
