<?php

/*
 *     doughnutwedding.com
 *     Copyright (C) 2017  http://doughnutwedding.com eric@doughnutwedding.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\DoughnutWeddingBundle\Repository;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\Query;
use Sikofitt\DoughnutWeddingBundle\Entity\User;
use Sikofitt\Security\TokenGenerator;
use Symfony\Component\Validator\Constraints\Email;
use Symfony\Component\Validator\Validation;

/**
 * Class UserRepository.
 *
 * Doctrine repository for User entity.
 */
class UserRepository extends EntityRepository
{
    /**
     * @param string $email
     *
     * @return null|User
     */
    public function findByEmail(string $email)
    {
        return $this->createQueryBuilder('u')
            ->select(['u', 'r'])
            ->leftJoin('u.rsvp', 'r', 'WITH', 'u.rsvp = r.id')
            ->where('u.email = :email')
            ->setParameter('email', $email)
            ->getQuery()
            ->getOneOrNullResult(Query::HYDRATE_OBJECT);
    }

    /**
     * @return int
     */
    public function getKatrinaCount()
    {
        return $this->createQueryBuilder('u')
            ->select('count(u.familySide)')
            ->distinct(true)
            ->where('u.familySide = :side')
            ->setParameter('side', User::KATRINA_SIDE)
            ->getQuery()
            ->getOneOrNullResult(Query::HYDRATE_SINGLE_SCALAR) ?? 0;
    }

    /**
     * @return int
     */
    public function getEricCount()
    {
        return $this->createQueryBuilder('u')
            ->select('count(u.familySide)')
            ->distinct(true)
            ->where('u.familySide = :side')
            ->setParameter('side', User::ERIC_SIDE)
            ->getQuery()
            ->getOneOrNullResult(Query::HYDRATE_SINGLE_SCALAR) ?? 0;
    }

    /**
     * @param string $email
     *
     * @return mixed|\Symfony\Component\Validator\ConstraintViolationListInterface
     */
    public function getEmail(string $email)
    {
        $validator = Validation::createValidator();
        $emailConstraint = new Email([
            'checkMX' => true,
            'checkHost' => true,
            'strict' => true,
        ]);

        $result = $validator->validate($email, [$emailConstraint]);

        if ($result->count() > 0) {
            return $result;
        }

        return $this->createQueryBuilder('u')
            ->select('u.email')
            ->where('u.email = :email')
            ->setParameter('email', $email)
            ->getQuery()
            ->getOneOrNullResult(Query::HYDRATE_SINGLE_SCALAR);
    }

    /**
     * @param string $resetToken
     *
     * @return User|null
     */
    public function getUserByResetToken(string $resetToken)
    {
        return $this->createQueryBuilder('u')
            ->select(['u', 'r'])
            ->leftJoin('u.rsvp', 'r', 'WITH', 'u.rsvp = r.id')
            ->where('u.resetToken = :resetToken')
            ->setParameter('resetToken', $resetToken)
            ->getQuery()
            ->getOneOrNullResult(Query::HYDRATE_OBJECT);
    }

    /**
     * @param string $userToken
     *
     * @return User|null
     */
    public function getUserByUserToken(string $userToken)
    {
        return $this->createQueryBuilder('u')
            ->select(['u', 'r'])
            ->leftJoin('u.rsvp', 'r', 'WITH', 'u.rsvp = r.id')
            ->where('u.userToken = :userToken')
            ->setParameter('userToken', $userToken)
            ->getQuery()
            ->getOneOrNullResult(Query::HYDRATE_OBJECT);
    }

    /**
     * Sets or updates user's user login token.
     *
     * @param string $email
     *
     * @return bool|string
     */
    public function setUserToken(string $email)
    {
        $userToken = (string) new TokenGenerator();

        $result = (bool) $this->createQueryBuilder('u')
            ->update()
            ->set('u.userToken', ':userToken')
            ->setParameter('userToken', $userToken)
            ->where('u.email = :email')
            ->setParameter('email', $email)
            ->getQuery()
            ->getOneOrNullResult(Query::HYDRATE_SINGLE_SCALAR);

        return true === $result ? $userToken : false;
    }

    /**
     * Sets or updates user's reset password token.
     *
     * @param string $email
     *
     * @return bool
     */
    public function setResetToken(string $email)
    {
        $resetToken = (string) new TokenGenerator();

        return (bool) $this->createQueryBuilder('u')
            ->update()
            ->set('u.resetToken', ':resetToken')
            ->setParameter('resetToken', $resetToken)
            ->where('u.email = :email')
            ->setParameter('email', $email)
            ->getQuery()
            ->getOneOrNullResult(Query::HYDRATE_SINGLE_SCALAR);
    }

    /**
     * Updates a user.
     *
     * @param User|\Symfony\Component\Security\Core\User\UserInterface $user
     */
    public function destoryUserResetToken(User $user)
    {
        $user->setResetToken(null);
        $this->getEntityManager()->persist($user);
        $this->getEntityManager()->flush();
    }
}
